import java.awt.*;
import javax.swing.*;

/**
 * A frame that draws a necklace. Can be refreshed.
 * @author Joel Ross
 * @version Fa 2014
 */
public class NecklaceFrame extends JFrame
{
	private static final int CONNECTOR_SIZE = 5; //the length of the connecting twine between beads

	private Necklace necklace;
	private NecklacePanel panel; //the panel we'll draw the necklace on (inner class)

	public NecklaceFrame(Necklace necklace)
	{
		super("Bead Necklace");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setResizable(false);

		this.necklace = necklace; //set the necklace

		panel = new NecklacePanel();
		this.add(panel);

		this.pack();
		this.setVisible(true);
	}

	/**
	 * A private inner class that contains the panel that we're drawing on.
	 * We've overwritten paintComponent to specify our own painting.
	 */
	private class NecklacePanel extends JPanel 
	{
		private static final int PANEL_HEIGHT = 50; //the height of the frame
		private static final int PANEL_WIDTH = 800; //the width of the frame;

		//constructor for necklace panel
		public NecklacePanel() 
		{
			this.setPreferredSize(new Dimension(PANEL_WIDTH,PANEL_HEIGHT));
		}

		//painting method
		public void paintComponent(Graphics g)
		{
			super.paintComponent(g);

			g.setColor(Color.WHITE); //white out the background
			g.clearRect(0, 0, PANEL_WIDTH, PANEL_HEIGHT);
			g.fillRect(0, 0, PANEL_WIDTH, PANEL_HEIGHT); //erase what has come before

			int x = 5; //current drawing spot
			for(int i=0; i<necklace.size(); i++)
			{
				Bead b = necklace.getBead(i);
				g.setColor(b.getColor());
				g.fillOval(x, PANEL_HEIGHT/2-b.getSize()/2, b.getSize(), b.getSize());
				x += b.getSize(); //move our current drawing spot
				//if(b.hook != null) //if want to not draw the "hook"
				//{
				g.setColor(Color.BLACK);
				g.drawLine(x, PANEL_HEIGHT/2, x+CONNECTOR_SIZE, PANEL_HEIGHT/2);
				x+= CONNECTOR_SIZE;
				//}
			}
		}
	}

	/**
	 * A method that "redraws" the frame to update the shown necklace.
	 */
	public void refreshNecklace()
	{
		panel.repaint();
	}
}
