import java.awt.Color;
import java.util.ArrayList;
import java.util.Random;

/**
 * A tester for the Necklace class
 * 
 * @author Joel Ross (adapted from Chuck Hommel)
 * @version Sp14
 */
public class NecklaceTester
{
	public static final int WAIT_TIME_MILLIS = 500;
	
	public static void main(String[] args)
	{
		//You should only run one of these methods at a time!

		makeConsistantNecklace(20);
		//makeNewNecklace();
		//removeFromFront();
		//randomRemove();

		//Palette.showPalette(); //to see the palette
	}

	/**
	 * Generates a "random" necklace that will always be the same. Helpful for testing add
	 */
	public static void makeConsistantNecklace(int numBeads)
	{
		ArrayList<Bead> theBeads = new ArrayList<Bead>();
		Random rand = new Random(2000); //makes a new random generator with the given starting "seed". 
											 //So the sequence of random numbers will always be the same
		Color[] colors = Palette.getPaletteColors();
		for (int i = 0; i < numBeads; i++)
		{ 
			Bead b = new Bead(colors[rand.nextInt(Palette.NUMBER_COLORS)], 10+rand.nextInt(20));
			theBeads.add(b);
		}

		Necklace neck = new Necklace();
		for(Bead bead : theBeads)
		{
			neck.addBead(bead);
		}
		NecklaceFrame frame = new NecklaceFrame(neck);

	}
	
	
	/**
	 * run test to add Beads to a necklace
	 */
	public static void makeNewNecklace()
	{
		Necklace neck = new Necklace();
		NecklaceFrame frame = new NecklaceFrame(neck);

		// test 1: add beads to necklace and display
		ArrayList<Bead> theBeads = new ArrayList<Bead>();
		for (int i = 0; i< 20; i++)
		{ 
			Bead b = new Bead();
			theBeads.add(b);
		}

		System.out.println("Running tests on " + theBeads.size() + " Beads.");
		System.out.println("*********************************\n");
		System.out.println("Test: Adding Beads...");
		System.out.println("*********************************\n");
		for (Bead b : theBeads)
		{
			System.out.println("Adding a Bead with color " + b.getColor());
			neck.addBead(b);
			frame.refreshNecklace();
			try
			{
				Thread.sleep(WAIT_TIME_MILLIS); //this pauses for a second, for nice animation
			}
			catch(InterruptedException e){}
		}
		System.out.print("Expected number of Beads in necklace: " + theBeads.size() + " ...");

		if (neck.size() == theBeads.size())
		{
			System.out.println("Pass.");
		}
		else
		{
			System.out.println("Necklace reports " + neck.size() + ". Fail.");
		}
	}

	/**
	 * Create a necklace and remove beads from front until empty.
	 */
	public static void removeFromFront()
	{
		// test 2: remove some beads, displaying
		Necklace neck = new Necklace();
		NecklaceFrame frame = new NecklaceFrame(neck);

		ArrayList<Bead> theBeads = new ArrayList<Bead>();
		for (int i = 0; i< 20; i++)
		{ 
			Bead b = new Bead();
			theBeads.add(b);
		}
		System.out.println("*********************************\n");
		System.out.println("Test: Removing Beads...");
		System.out.println("*********************************\n");
		for (Bead b : theBeads)  // make a necklace 
		{
			neck.addBead(b);
		}

		frame.refreshNecklace();
		System.out.println("Testing removing all Beads from front.");
		Color[] testColors = Palette.getPaletteColors();
		int numRemoved = 0;
		int expectedCount = theBeads.size();
		for (Color c : testColors)
		{
			int count = 0;
			System.out.println("Removing Beads of color : " + c + " ...");
			Bead b = neck.removeBead(c);
			while (b != null)
			{
				if (b.getColor().getRGB() != c.getRGB())
				{
					System.out.println("Incorrect color Bead returned.");
				}
				else
				{
					System.out.println("Correct Bead returned.");
					count++;
				}
				frame.refreshNecklace();
				try
				{
					Thread.sleep(WAIT_TIME_MILLIS); //this pauses for a second, for nice animation
				}
				catch(InterruptedException e){}
				b = neck.removeBead(c);
			}
			System.out.println("Done. " + count + " Beads removed.");
			numRemoved += count;
		}
		System.out.println("Expected to remove " + expectedCount + " Beads.");
		System.out.print("Actually removed " + numRemoved + " Beads...");
		if (expectedCount == numRemoved)
		{
			System.out.println("Pass. ");
		}
		else
		{
			System.out.println("Fail. ");
		}
	}

	/**
	 * Randomly remove Beads from necklace
	 */
	private static void randomRemove()
	{
		Necklace neck = new Necklace();
		NecklaceFrame frame = new NecklaceFrame(neck);

		ArrayList<Bead> theBeads = new ArrayList<Bead>();
		for (int i = 0; i< 20; i++)
		{ 
			Bead b = new Bead();
			theBeads.add(b);
		}
		System.out.println("*********************************\n");
		System.out.println("Test: Randomly Removing Beads...");
		System.out.println("*********************************\n");
		for (Bead b : theBeads)  // make a necklace
		{
			neck.addBead(b);
		}

		frame.refreshNecklace();
		System.out.println("Testing removing Beads randomly.");
		Color[] testColors = Palette.getPaletteColors();
		int numRemoved = 0;
		int startingCount = neck.size();
		Random randGen = new Random();   // generate random indices to testColors
		for (int i = 0; i < 20; i++ )  // run 20 tests)
		{
			int count = 0;
			Color c = testColors[randGen.nextInt(testColors.length)];
			System.out.println("Removing Bead of color : " + c + " ...");
			Bead b = neck.removeBead(c);
			if (b != null)
			{
				if (b.getColor().getRGB() != c.getRGB())
				{
					System.out.println("Incorrect color Bead returned.");
				}
				else
				{
					System.out.println("Correct Bead returned.");
					count++;
				}
			}
			else
			{
				System.out.println("No Bead returned.");
			}
			frame.refreshNecklace();
			try
			{
				Thread.sleep(WAIT_TIME_MILLIS);
			}
			catch(InterruptedException e){}
			System.out.println("Done. " + count + " Beads removed.");
			numRemoved += count;
		}

		System.out.println("Initial necklace size: " + startingCount);
		System.out.println("Final necklace size:  " + neck.size());
		System.out.print("Actually removed " + numRemoved + " Beads...");        

		if (numRemoved + neck.size()  == startingCount)
		{
			System.out.println("Pass. ");
		}
		else
		{
			System.out.println("Fail. ");
		}

	}

}
